#include <winsock2.h>
#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <errno.h>
#include <crtdbg.h>
#include <OCIdl.h>
#include <Olectl.h>

#include "cresource.h"


#pragma comment(lib, "user32")
#pragma comment(lib, "ws2_32")
#pragma comment(lib, "gdi32")
#pragma comment(lib, "ole32")
#pragma comment(lib, "oleaut32")

LPCSTR GetIniFile()
{
	static char path[MAX_PATH] = {0};
	GetModuleFileName(GetModuleHandle(NULL), path, MAX_PATH);
	LPSTR p = strrchr(path, '.');
	if (p)
	{
		strcpy(p, ".ini");
	}

	return path;
}

LPCSTR GetAddress()
{
	static char addr[256] = {0};
	GetPrivateProfileStringA("server", "address", "127.0.0.1", addr, sizeof(addr), GetIniFile());
	return addr;
}

typedef struct Init
{
	HRESULT hr;
    WSADATA wsaData;
    int     res;

	Init()
	{
		hr = CoInitialize(NULL);
	    res = WSAStartup(MAKEWORD(2,2), &wsaData);
	}

	~Init()
	{
	    WSACleanup();
		if (SUCCEEDED(hr))
			CoUninitialize();
	}

	BOOL isOkay()
	{
	    return (res == NO_ERROR) && SUCCEEDED(hr);
	}
} Init;

typedef struct StreamData
{
	HGLOBAL hGlobal;
	LPBYTE  data;
	ULONG   size;
	StreamData(IStream* pStream)
	{
		hGlobal = NULL;
		HRESULT hr = GetHGlobalFromStream(pStream, &hGlobal);
		_ASSERT(SUCCEEDED(hr));
		size = (ULONG)GlobalSize(hGlobal);
		data = (LPBYTE)GlobalLock(hGlobal);
	}
	~StreamData() 
	{
		GlobalUnlock(hGlobal);
	}
} StreamData;

typedef struct TxSOCKET
{
    int res;
	int wsaErr;
    SOCKET sock;
	bool connected;
    struct sockaddr_in srv_in; 
	static const int nPort = 15777;

	TxSOCKET(): 
		sock(INVALID_SOCKET),
		connected(false), 
		res(SOCKET_ERROR), 
		wsaErr(0)
	{
	}
	~TxSOCKET() { close(); }

	bool connect()
	{
		sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
		if (sock == INVALID_SOCKET) 
			return false;

		srv_in.sin_family = AF_INET;
		srv_in.sin_addr.s_addr = inet_addr( GetAddress() );
		srv_in.sin_port = htons( nPort );

		res = ::connect( sock, (SOCKADDR*) &srv_in, sizeof(srv_in) );
		if (res == SOCKET_ERROR) 
		{
			close();
			sock = INVALID_SOCKET;
		}
		else
			connected = true;

		return (sock != INVALID_SOCKET);
	}

	void close()
	{
		if (sock != INVALID_SOCKET)
		{
			closesocket(sock);
			connected = false;
			sock = INVALID_SOCKET;
		}
	}

	int sendBuffer(LPBYTE buf, ULONG size)
	{
	    int res = ::send( sock, (LPCSTR)buf, size, 0 );
	    if (res == SOCKET_ERROR) 
	    {
	        close();
	    }

		res = shutdown(sock, SD_SEND);
		if (res == SOCKET_ERROR) 
		{
			close();
		}

		char rbuf[20] = {0};
        res = ::recv(sock, rbuf, sizeof(rbuf), 0);
        if ( res > 0 && 0 == _tcscmp(TEXT("OK"), rbuf))
		{
			wsaErr = 0;
		}
        else if ( res == 0 )
            close();
        else
            wsaErr = WSAGetLastError();

		return res;
	}
} TxSOCKET;

Init init;
HBITMAP hbm = NULL;
IStream* pStream = NULL;

BOOL Capture(HWND hDlg)
{
//	hDlg = GetDesktopWindow();
	HDC dc = GetDC(hDlg);
	HDC cdc = CreateCompatibleDC(dc);
	DeleteObject((HGDIOBJ)hbm);
	RECT rc = {0};
	GetClientRect(hDlg, &rc);
	hbm = CreateCompatibleBitmap(dc, rc.right, rc.bottom);
	BITMAP bm = {0};
	if (hbm && GetObject(hbm, sizeof(bm), &bm))
	{
		HGDIOBJ oldBm = SelectObject(cdc, (HGDIOBJ)hbm);
		BitBlt(cdc, 0, 0, bm.bmWidth, bm.bmHeight, dc, 0, 0, SRCCOPY);
		SelectObject(cdc, oldBm);
	}
	DeleteDC(cdc);
	ReleaseDC(hDlg, dc);
	return (hbm != NULL);
}

LONG Save()
{
	LONG pictSize = 0;
	IPicture* pPict = NULL;
	PICTDESC pd = { sizeof(pd), PICTYPE_BITMAP };
	pd.bmp.hbitmap = hbm;

	HRESULT hr = ::OleCreatePictureIndirect(&pd, IID_IPicture, TRUE, (void**)&pPict);
	if (SUCCEEDED(hr))
	{
		_ASSERT(pPict);
		if (pStream)
		{
			pStream->Release();
		}
		hr = ::CreateStreamOnHGlobal(NULL, TRUE, &pStream);
		if (SUCCEEDED(hr))
		{
			hr = pPict->SaveAsFile(pStream, TRUE, &pictSize);
			_ASSERT(SUCCEEDED(hr));
		}
	}
	else
	{
		_ASSERT(FALSE);
	}
	if (pPict)
		pPict->Release();

#ifdef DEBUG
	FILE* f = fopen("client.bmp", "w");
	if (f)
	{
		StreamData image(pStream);
		_ASSERT(image.size == pictSize);
		fwrite(image.data, 1, image.size, f);
	}
	fclose(f);
#endif
	return pictSize;
}

BOOL OnInitDialog(HWND hDlg, WPARAM wp, LPARAM lp)
{
	SetTimer(hDlg, 1, 200, NULL);
	return TRUE;
}

void OnPaint(HWND hDlg)
{
	int x = 10, y = 10;
	SYSTEMTIME st = {0};
	GetLocalTime(&st);
	TCHAR timeStr[20] = {0};
	wsprintf(timeStr, TEXT("%02d:%02d:%02d"), st.wHour, st.wMinute, st.wSecond);
	PAINTSTRUCT ps;
	HDC dc = BeginPaint(hDlg, &ps);
	TextOut(dc, x, y, timeStr, _tcslen(timeStr));
	EndPaint(hDlg, &ps);
}


BOOL CALLBACK DlgProc(HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
{
	switch (msg)
	{
	case WM_INITDIALOG:
		return OnInitDialog(hDlg, wp, lp);

	case WM_PAINT:
		OnPaint(hDlg);
		break;

	case WM_TIMER:
		{
			if (IsDlgButtonChecked(hDlg, IDC_AUTOUPDATE))
				SendMessage(hDlg, WM_LBUTTONUP, 0, 0);
			InvalidateRect(hDlg, NULL, TRUE);
			break;
		}

	case WM_LBUTTONUP:
		{
			if (Capture(hDlg))
			{
				LONG size = Save();
				if (size)
				{
					TxSOCKET tx;
					if (tx.connect())
					{
						StreamData image(pStream);
						int res = tx.sendBuffer(image.data, image.size);
						_ASSERT(res > 0);
					}
				}
			}
			else
				MessageBox(hDlg, TEXT("Capture failed"), TEXT("Oops!"), MB_OK);
			break;
		}

	case WM_COMMAND:
		switch (LOWORD(wp))
		{
		case IDCANCEL:
			EndDialog(hDlg, 0);
			break;
		}
		break;
	}
	return 0;
}

int WINAPI _tWinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmd, INT nShow)
{
	if (!init.isOkay())
	{
		TCHAR errMsg[512] = {0};
		wsprintf(errMsg, TEXT("res = %d, hr = 0x%08x"), init.res, init.hr);
		MessageBox(NULL, errMsg, TEXT("Initialization failure"), MB_OK | MB_ICONHAND);
		return 1;
	}

	int res = DialogBox(hInst, MAKEINTRESOURCE(IDD_MAIN), NULL, (DLGPROC)DlgProc);

	return 0;
}
